@@ -5,9 +5,9 @@
     require Carp;
 
     @EXPORT = qw(); # Do NOT @EXPORT anything.
-    $VERSION = sprintf("%d.%02d", q$Revision: 11.4 $ =~ /(\d+)\.(\d+)/o);
+    $VERSION = sprintf("%d.%02d", q$Revision: 11.6 $ =~ /(\d+)\.(\d+)/o);
 
-#   $Id: Sponge.pm,v 11.4 2002/05/22 13:14:13 timbo Exp $
+#   $Id: Sponge.pm,v 11.6 2002/10/29 10:00:44 timbo Exp $
 #
 #   Copyright (c) 1994, Tim Bunce
 #
@@ -39,7 +39,6 @@
     $imp_data_size = 0;
     # we use default (dummy) connect method
     sub disconnect_all { }
-    sub DESTROY { }
 }
 
 
@@ -142,8 +141,6 @@
         return $dbh->SUPER::STORE($attrib, $value);
     }
 
-    sub DESTROY { }
-
 }
 
 
@@ -163,7 +160,7 @@
 	    my $NUM_OF_PARAMS = $sth->{NUM_OF_PARAMS};
 	    return $sth->set_err(1, @$row." values bound (@$row) but $NUM_OF_PARAMS expected")
 		if @$row != $NUM_OF_PARAMS;
-	    $sth->trace_msg("inserting (@$row)\n");
+	    { local $^W; $sth->trace_msg("inserting (@$row)\n"); }
 	    push @{ $sth->{rows} }, $row;
 	}
 	# else do nothing for select as data is already in $sth->{rows}
@@ -194,8 +191,6 @@
 	# else pass up to DBI to handle
 	return $sth->SUPER::STORE($attrib, $value);
     }
-
-    sub DESTROY { }
 }
 
 1;
